#ifndef _CONFIGURATION_H_
#define _CONFIGURATION_H_

/* The Pre-Defined Configuration
*
* Configuration.h
* fwk4gps version 1.0
* gam666/dps901
* August 30 2010
* copyright (c) 2010 Chris Szalwinski 
* distributed under TPL - see ../Licenses.txt
*/

// Timing Factors
//
// units of time per sec returned by timeGetTime()
#define UNITS_PER_SEC   1000
// fps maximum - should be > flicker fusion threshold
#define FPS_MAX          200
// minimum elapsed time for updating to the next frame
#define MIN_ELAPSED_TIME UNITS_PER_SEC/FPS_MAX

// Window Dimensions
#define WND_WIDTH  1280   // minimum window width
#define WND_HEIGHT 800   // minimum window height

// Sound Directory
#define AUDIO_DIRECTORY L"resources\\audio"

// Texture Directory
#define TEXTURE_DIRECTORY L"resources\\textures"

// NULL address
#define NULL 0

// Maximum string length throughout
#define MAX_DESC 255

// Sizes of context arrays
//
#define MAX_INTEGERS     35
#define MAX_VECTORS       4
#define MAX_BOOL_ARRAYS   3
#define MAX_INT_ARRAYS    4
#define MAX_STR_ARRAYS    8
#define MAX_ADAPTERS     10
#define MAX_MODES        50
#define MAX_P_FORMATS    20
#define MAX_KEYS        256
#define MAX_M_BUTTONS     3
#define MAX_C_BUTTONS   128
#define MAX_CONTROLLERS  10

typedef enum Integer {
    GF_AU_FREQ,
    GF_AU_MXVL,
    GF_AU_MNVL,
    GF_AU_CVOL,
    GF_DS_ACNT,
    GF_DS_MCNT,
    GF_DS_PCNT,
    GF_DS_ADPT,
    GF_DS_MODE,
    GF_DS_PIXL,
    GF_DS_RINW,
    GF_DS_WDTH,
    GF_DS_HGHT,
    GF_DS_MXLT,
    GF_DS_PXFW,
    GF_WN_WDTH,
    GF_WN_HGHT,
    GF_MS_POSX,
    GF_MS_POSY,
    GF_MS_ORIZ,
    GF_CT_INDX,
    GF_CT_TGRB,
    GF_CT_FLGS,
    GF_CT_CCNT,
    GF_CT_BCNT,
    GF_CT_DSPX,
    GF_CT_DSPY,
    GF_CT_DSPZ,
    GF_CT_ROTZ,
    GF_NO_ACTS,
    GF_NO_KEYS,
    GF_NO_SNDS,
    GT_XX_XXXX
} Integer;

typedef enum VectorId {
    GF_CA_POSN,
    GF_CA_HEAD,
    GF_CA_UP,
    GF_LT_BKGD
} VectorId;

typedef enum BoolArray {
    GF_KB_KEYS, // key states
    GF_MS_BTNS, // mouse button states
    GF_CT_BTNS,  // controller button states
    LOCK_MOVE
} BoolArray;

typedef enum IntArray {
    GF_TRGRBUTN,  // trigger buttons
    GF_KEYCODES  // key codes
} IntArray;

typedef enum StrArray {
    GF_ADPDESC, // adapter descriptions
    GF_MDEDESC, // mode descriptions
    GF_CTRDESC, // controller descriptions
    GF_CTBDESC, // controller button descriptions
    GF_ACTDESC, // action descriptions
    GF_KEYDESC, // key descriptions
    GF_SNDDESC, // sound descriptions
    GF_SNDFILE  // sound file names
} StrArray;

// mappable keys for configurable actions (do not initialize these constants)
//
typedef enum Key { 
    KEY_A,
    KEY_B,
    KEY_C,
    KEY_D,
    KEY_E,
    KEY_F,
    KEY_G,
    KEY_H,
    KEY_I,
    KEY_J,
    KEY_K,
    KEY_L,
    KEY_M,
    KEY_N,
    KEY_O,
    KEY_P,
    KEY_Q,
    KEY_R,
    KEY_S,
    KEY_T,
    KEY_U,
    KEY_V,
    KEY_W,
    KEY_X,
    KEY_Y,
    KEY_Z,
    KEY_1,
    KEY_2,
    KEY_3,
    KEY_4,
    KEY_5,
    KEY_6,
    KEY_7,
    KEY_8,
    KEY_9,
    KEY_0,
    KEY_F1,
    KEY_F2,
    KEY_F3,
    KEY_F4,
    KEY_F5,
    KEY_F6,
    KEY_F7,
    KEY_F8,
    KEY_F9,
    KEY_F10,
    KEY_F11,
    KEY_F12,
    KEY_SPACE,
    KEY_ENTER,
    KEY_UP   ,
    KEY_DOWN, 
    KEY_PGUP, 
    KEY_PGDN, 
    KEY_LEFT, 
    KEY_RIGHT,
    KEY_NUM1,
    KEY_NUM2,
    KEY_NUM3,
    KEY_NUM4,
    KEY_NUM5,
    KEY_NUM6,
    KEY_NUM7,
    KEY_NUM8,
    KEY_NUM9,
    KEY_ESCAPE,
    KEY_SEMICOLON,
    KEY_APOSTROPHE,
    KEY_O_BRACKET,
    KEY_C_BRACKET,
    KEY_BACKSLASH,
    KEY_COMMA,
    KEY_PERIOD,
    KEY_SLASH,
    KEY_TIMES,
    KEY_GRAVE,
    KEY_MINUS,
    KEY_UNDERSCORE,
    KEY_EQUALS,
    KEY_PLUS
} Key;

// mappable keys for configurable actions as listed in the user dialog
//
#define KEY_DESCRIPTIONS {\
    L"A", L"B", L"C", L"D", L"E", L"F", L"G", L"H", L"I", L"J", \
    L"K", L"L", L"M", L"N", L"O", L"P", L"Q", L"R", L"S", L"T", \
    L"U", L"V", L"W", L"X", L"Y", L"Z", \
    L"1", L"2", L"3", L"4", L"5", L"6", L"7", L"8", L"9", L"0", \
    L"F1", L"F2", L"F3", L"F4", L"F5", L"F6", L"F7", \
    L"F8", L"F9", L"F10", L"F11", L"F12", \
    L"Space", L"Enter", L"Up", L"Down", L"PageUp", L"PageDown", \
    L"Left", L"Right", \
    L"NumPad 1", L"NumPad 2", L"NumPad 3", L"NumPad 4", L"NumPad 5", \
    L"NumPad 6", L"NumPad 7", L"NumPad 8", L"NumPad 9", \
    L"Escape", L";", L"'", L"[", L"]", L"\\", L",", \
    L".", L"/", L"*", L"`", L"-", L"_", L"=", L"+" \
    }

// Actions
//
// to add an action
// - add the enumeration constant for the new action
// - add the friendly description of the new action
// - reset MAX_DESC above if necessary 
// - add the default key for the new action
//
// enumeration constants
typedef enum Action {
    MOVE_UP,
    MOVE_DOWN,
    MOVE_LEFT,
    MOVE_RIGHT,
    HUD_DISPLAY,
    TOGGLE_FLASH,
    MOVE_BOX,
    CHANGE_BOX,
    DISPLAY_BOX,
    OPTION_ONE,
    OPTION_TWO,
    OPTION_THREE,
    OPTION_FOUR

} Action;

// user friendly descriptions of actions
//
#define ACTION_DESCRIPTIONS {\
    L"Roll Boxes",   \
    L"Spin Left Box",\
    L"Spin Top Box", \
    L"Clone Right Box", \
    L"Clone Left Sound", \
    L"Clone SpotLight", \
    L"Pitch Up",     \
    L"Pitch Down",   \
    L"Yaw Left",     \
    L"Yaw Right",    \
    L"Move Camera Forward",     \
    L"Move Camera Backward",    \
    L"Toggle Background Sound", \
    L"Explosion Sound", \
    L"Increase Volume", \
    L"Decrease Volume", \
    L"Speed Up Sound",  \
    L"Slow Down Sound", \
    L"Toggle Point Light",   \
    L"Toggle Spot Light",    \
    L"Toggle Distant Light", \
    L"Toggle Cloned Light", \
    L"Toggle Heads Up Display",      \
    L"Heads Up Display - Move Left", \
    L"Heads Up Display - Move Right",\
    L"Heads Up Display - Move Up",   \
    L"Heads Up Display - Move Down",  \
    L"Select Camera" \
    }

// initial mappings of actions to keys
//
#define ACTION_MAPPINGS {KEY_W, KEY_S, KEY_A, KEY_D,\
    KEY_F, KEY_Q,\
    KEY_T, KEY_Y, KEY_R,\
    KEY_1,KEY_2,KEY_3,KEY_4 }

// Mouse Buttons
//
typedef enum MouseButton {
    LEFT_BUTTON,
    RIGHT_BUTTON
} MouseButton;

// Controller Buttons
//
typedef enum CntlrButton {
    TRIGGER,
    BUTTON_1,
    BUTTON_2,
    BUTTON_3,
    BUTTON_4,
    BUTTON_5,
    BUTTON_6,
    BUTTON_7,
    BUTTON_8,
    BUTTON_9,
    BUTTON_10,
    NORTH,
    EAST,
    WEST,
    SOUTH
} CntrlrButton;

// Graphics Primitive Types
//
typedef enum PrimitiveType {
    POINT_LIST     = 1,
    LINE_LIST      = 2,
    LINE_STRIP     = 3,
    TRIANGLE_LIST  = 4,
    TRIANGLE_STRIP = 5,
    TRIANGLE_FAN   = 6
} PrimitiveType;

// Sound Types
//
typedef enum SoundType {
    LOCAL_SOUND,  // stationary 3D
    MOBILE_SOUND, // mobile 3D
    GLOBAL_SOUND  // ambient
} SoundType;

// Light Types
//
typedef enum LightType {
    POINT_LIGHT,
    SPOT_LIGHT,
    DIRECTIONAL_LIGHT
} LightType;

// Model Sounds
//
// to add a configurable sound
// - add its enumeration constant
// - add its description
// - add the default filename for the sound
// - reset MAX_DESC in Configuration.h if necessary 
//
typedef enum ModelSound {
    SND_BKGRD,
    SND_DISCR,
    SND_OBJECT,
    SND_LOCAL_L,
    SND_LOCAL_R,
    SND_BKGRD_LOSE
} ModelSound;

// friendly descriptions of configurable sounds as listed in the user dialog 
//
#define SOUND_DESCRIPTIONS {\
    L"Background",\
    L"Discrete",\
    L"Object",\
    L"Local Left",\
    L"Local Right",\
    L"Game Over"\
    }

// initial selection of configurable sounds 
//
// include the authors name for CCS+ accreditation
//
#define SOUND_MAPPINGS {\
    L"background.wav",\
    L"corpsehere__Zombieme.wav",\
    L"Corsica_S_zombie_news_slower.wav",\
    L"aust_paul__rip_tear_FLESH_.wav",\
    L"corpsehere__Zombieme.wav",\
    L"gameover.wav"\
    }

// Dialog options
//
// default adapter selection
#define RUN_IN_WINDOW_DESC L"Run in a window"
#define RESOLUTION_DESC    L""
// default controller
#define NO_CONTROLLER_DESC L"No Controller"
#define TRIGGER_DESC       L""

// Text alignment flags
//
#define TXT_LEFT    1
#define TXT_RIGHT   2
#define TXT_CENTER  4
#define TXT_TOP     8
#define TXT_BOTTOM 16
#define TXT_MIDDLE 32
#define TXT_DEFAULT TXT_LEFT | TXT_TOP

// Texture filtering flags
//
#define TEX_MIN_POINT        1
#define TEX_MIN_LINEAR       2
#define TEX_MIN_ANISOTROPIC  4
#define TEX_MAG_POINT        8
#define TEX_MAG_LINEAR      16
#define TEX_MAG_ANISOTROPIC 32
#define TEX_DEFAULT TEX_MIN_LINEAR | TEX_MAG_LINEAR

// sprite modulation colour for the HUD texture [0,255]
//
#define SPRITE_R 255
#define SPRITE_G 255
#define SPRITE_B 255

// display background colour
//
#define BGROUND_R 200
#define BGROUND_G 200
#define BGROUND_B 200

// projection frustum properties
//
#define NEAR_CLIPPING 1.0f   // in model units
#define FAR_CLIPPING  2000.f // in model units 
#define FIELD_OF_VIEW 0.9f   // in radians

// font display colour [0,255]
//
#define TXT_R 255
#define TXT_G 255
#define TXT_B 255
#define TXT_A 255

// sound parameters
//
#define INTERIOR_CONE               0.67f // in radians
#define SOUND_DISTANCE_FACTOR       1.0f  // metres per model unit
#define ATTENUATION_DISTANCE_FACTOR 10    // where attenuation begins

#endif


