/* Design Implementation - Design Component - Model Branch
*
* Design.cpp
* fwk4gps version 1.0
* gam666/dps901
* August 30 2010
* copyright (c) 2010 Chris Szalwinski 
* distributed under TPL - see ../Licenses.txt
*/

#include "iContext.h"        // for the Context Interface
#include "iText.h"           // for the Text Interface
#include "iSound.h"          // for the Sound Interface
#include "iLight.h"          // for the Light Interface
#include "iObject.h"         // for the Object Interface
#include "iTexture.h"        // for the Texture Interface
#include "iCamera.h"         // for the Camera Interface
#include "iUtilities.h"      // for strcpy()
#include "iFrame.h" 
#include "Design.h"          // for the Design class declaration
#include "MathDefinitions.h" // for MODEL_Z_AXIS
#include "ModelSettings.h"   // for FLOOR, MOUSE_BUTTON_SCALE, ROLL_SPEED
#include <stdlib.h>
#define MIN_ROTATE 200
#define AFTEREND 10000
#define CHANGE_CIRCLE 100
//-------------------------------- Design -------------------------------------
//
// The Design Component implements the design within the model branch
//
// CreateDesign creates the Design Object
//
iDesign* CreateDesign(iContext* c) {

    return new Design(c);
}

// constructor initializes the object and texture pointers and the reference
// time
//
Design::Design(iContext* c) : context(c) {

    // pointers to the model sounds
    background   = NULL;
    discrete     = NULL;
    locall       = NULL;
    localr       = NULL;

    // pointers to the model lights
    puzzleLight1  = NULL;
    spotLight    = NULL;
    puzzleLight2 = NULL;
    puzzleLight3   = NULL;
    puzzleLight4 = NULL;

    // pointers to the model objects
    puz_cylinder1= NULL;
    puz_cylinder2= NULL;
    puz_cylinder3= NULL;
    rollRight    = NULL;
    spinTop      = NULL;
    riddleQ      = NULL;

    // initialize reference time
    lastUpdate = 0;
    changeOne=0;
    showpuzzle=0;
    randompos[0]=0;
    randompos[1]=0;
    randompos[2]=0;
    solved[0]=0;
    solved[1]=0;
    solved[2]=0;
    section=0;
    firstTime[0]=true;
    firstTime[1]=true;
    firstTime[2]=true;
    answered[0]=false;
    answered[1]=false;
    answered[2]=false;
    end = false;
    after_end = false;
    whichRiddle=0;
    puzzle1_light = Vector(205, 95, 69);
    puzzle2_light = Vector(241, 95, 1011);
    puzzle3_light = Vector(1185, 95, 1887);
    puzzle4_light = Vector(371, 95, -332);
    lastRiddle= puzzle4_light;
    emergencyLight = NULL;
    after=false;
    elapsed=0;
    gameover=false;
}

// initialize creates the objects and textures in the initial scene
// and initializes the reference time
//
void Design::initialize(int now) {

    // ------------------------------------------------------------------------
    // audio ------------------------------------------------------------------
    // ------------------------------------------------------------------------

    // create background sound
    if (context->get(GF_SNDFILE, SND_BKGRD))
        background = CreateSound(context->get(GF_SNDFILE, SND_BKGRD), GLOBAL_SOUND, true, true);

     //Game Won
    if (context->get(GF_SNDFILE, SND_OBJECT))
        discrete = CreateSound(context->get(GF_SNDFILE, SND_OBJECT), GLOBAL_SOUND, true, false, 500);
    //Game Lost
    if (context->get(GF_SNDFILE, SND_BKGRD_LOSE))
        game_over = CreateSound(context->get(GF_SNDFILE, SND_BKGRD_LOSE), GLOBAL_SOUND, true, false, 500);


    //Flesh tearing
    if (context->get(GF_SNDFILE, SND_LOCAL_L)) {
        locall = CreateSound(context->get(GF_SNDFILE, SND_LOCAL_L), 
            LOCAL_SOUND, true, true, 500);
        locall->translate(-143, 90, 551 * MODEL_Z_AXIS);
        locall->rotatey(3.14f);
    }

    //Zombie Moan
    if (context->get(GF_SNDFILE, SND_LOCAL_R)) {
        localr = CreateSound(context->get(GF_SNDFILE, SND_LOCAL_R), LOCAL_SOUND, true, true, 500);
        localr->translate(1415, 90, 1286 * MODEL_Z_AXIS);
        localr->rotatey(3.14f);
    }   

    // ------------------------------------------------------------------------
    // lighting ---------------------------------------------------------------
    // ------------------------------------------------------------------------

    Colour white(1, 1, 1);
    Colour yellow(0.7f, 0.7f, 0.7f);
    Colour turquoise(0, 0.8f, 0.6f);
    Colour red(1, 0, 0);
    Colour black(0, 0, 0);

    context->set(GF_LT_BKGD,
        Vector(0.09f,0.09f,0.09f));


    //Pointlights
    puzzleLight1 = CreatePointLight(white,white ,white ,context, 150.0f, true);
    puzzleLight1->translate(puzzle1_light.x,puzzle1_light.y,puzzle1_light.z);
    puzzleLight2 = CreatePointLight(white,white ,white ,context, 150.0f, true);
    puzzleLight2->translate(puzzle2_light.x,puzzle2_light.y,puzzle2_light.z);
    puzzleLight3 = CreatePointLight(white,white ,white ,context, 150.0f, true);
    puzzleLight3->translate(puzzle3_light.x,puzzle3_light.y,puzzle3_light.z);
    puzzleLight4 = CreatePointLight(white,white ,white ,context, 300.0f, false);
    puzzleLight4->translate(puzzle4_light.x,puzzle4_light.y,puzzle4_light.z);
    emergencyLight = CreatePointLight(red,red ,red ,context, 40.0f, true);
    emergencyLight->translate(510, 180, -336);

    // Create a directional light
    //
    distantLight = CreateDistantLight(red, red, red,context, false);
    distantLight->translate(175, 300, 75);


    //Flashlight
    topLight = CreateSpotLight(white, white, white,context, 500.0f, true, 
        0, 0.005f, 0, ToRadians(40.0f), ToRadians(20.0f), 1);


    // --------------------------------------------------------------------------
    // Objects ------------------------------------------------------------------
    // --------------------------------------------------------------------------

    // create textures
    iTexture* zombie_base = CreateTexture(L"zombie_1.bmp");
    iTexture* zombie_middle = CreateTexture(L"zombie_2.bmp");
    iTexture* zombie_top = CreateTexture(L"zombie_3.bmp");
    iTexture* checkdsy = CreateTexture(L"black-background.bmp");
    iTexture* solution1 = CreateTexture(L"solved_1.bmp");
    iTexture* riddle = CreateTexture(L"riddle1.bmp");

    camera= CreateCamera(context);

    // initial objects
    rollRight = CreateBox(0, 0, 0 , WND_WIDTH/16, WND_HEIGHT/16, 1, yellow);
    rollRight->attach(checkdsy);
    rollRight->translate(-180,500, -90);
    rollRight->rotatex(ToRadians(180));
    rollRight->rotatey(ToRadians(120));

    puz_cylinder1 = CreateCylinder(17, .1f, 100,yellow); 
    puz_cylinder1->attach(zombie_base);
    puz_cylinder1->translate(-23, 500, 30 * MODEL_Z_AXIS);
    puz_cylinder1->rotatex(ToRadians(90));
    baby = false;
    pyramid = true;

    puz_cylinder2 = CreateCylinder(10, 1, 100,yellow); 
    puz_cylinder2->attach(zombie_middle);
    puz_cylinder2->translate(-23, 500, 30 * MODEL_Z_AXIS);
    puz_cylinder2->rotatex(ToRadians(90));
    
    puz_cylinder3 = CreateCylinder(5, 1, 100,yellow); 
    puz_cylinder3->attach(zombie_top);
    puz_cylinder3->translate(-23, 500, 30 * MODEL_Z_AXIS);
    puz_cylinder3->rotatex(ToRadians(90));
    
    puz_cylinder1->rotatey(ToRadians(120));
    puz_cylinder2->rotatey(ToRadians(120));
    puz_cylinder3->rotatey(ToRadians(120));

    spinTop = CreateBox(0, 0, 0 , 32, 32, 1, yellow);
    spinTop->attach(solution1);
    spinTop->translate(-200,500, -90);
    spinTop->rotatex(ToRadians(180));
    spinTop->rotatey(ToRadians(120));

    riddleQ = CreateBox(0, 0, 0 , 32, 32, 1, yellow);
    riddleQ->translate(-200,500, -90);
    riddleQ->rotatex(ToRadians(180));
    riddleQ->rotatey(ToRadians(120));

    context->set(LOCK_MOVE,0,false);
    // camera at a distance - in lhs coordinates
    camera->translate(-145, 90, -50);

    //Displays an information textbox
    CreateText(0.02f, 0.2f, 1, 0.99f, TXT_CENTER, L"Welcome to the zombie game\n\nYour character is a scientist in a secret underground military lab.\n"
        L"Youve been working on a cure for Aids, and your latest vaccine seems to be working \n"
        L"unfortunately it has some adverse side effects.After about one week your test subjects begin\n"
        L"to mutate. Their heart stops, their body starts to decompose, their frontal lobes disintegrate,\n"
        L"yet instead of dying they begin to act solely on instinct. The vaccine has also mutated\n"
        L"that when an individual comes into contact with the affected individuals saliva they will\n"
        L"begin to mutate as well. The mutation quickly runs through your laboratory and only you\n"
        L"are left to escape. You must solve all the puzzles to unlock the exit door and earn your freedom.\n\n"
        L"Keys:\n   Movement:                                      Toggle light: Q\n"
        L"   W - Forward                                    Info textbox toggle: F\n"
        L"   S - Backward                                  Display/hide the puzzle/riddle: R\n"
        L"   A - Move left                                     Rotate puzzle section: T\n"
        L"   D - Move right                                 Change puzzle section: Y\n"
        L"   Freelook: Mouse                            Answer the riddle: 1 to 4");

    // initialize reference time ----------------------------------------------
    //
    lastUpdate = now;
}

// reset changes the context if any change has been requested
// and reinitializes the reference time
//
void Design::reset(int now) {

    // reset reference time ---------------------------------------------------
    //
    showpuzzle=0;
    solved[0]=0;
    solved[1]=0;
    solved[2]=0;
    section=0;
    firstTime[0]=true;
    firstTime[1]=true;
    firstTime[2]=true;
    answered[0]=false;
    answered[1]=false;
    answered[2]=false;
    end = false;
    after_end = false;
    gameover=false;
    context->set(LOCK_MOVE,0,false);
    whichRiddle=0;
    rollRight->translate(0,500,0);
    puz_cylinder1->translate(0, 500, 0);
    puz_cylinder2->translate(0,500, 0);
    puz_cylinder3->translate(0, 500, 0);
    spinTop->translate(0, 500, 0);
    lastUpdate = now;
}

// restore re-initializes the reference time
//
void Design::restore(int now) {

    lastUpdate = now;
}

// setupPuzzle method
// Displays the puzzle for the user.
// Depending on which position the user is at, it will display the proper display.
// The user could hide the puzzle and return to it.
// If the user solved the puzzle already, it would display the completed message.
//
void Design::setupPuzzle(Vector camerapos) {

    //Stores position of the puzzles currently
    Vector box = rollRight->position();
    Vector cylinder_base = puz_cylinder1->position();
    Vector cylinder_middle = puz_cylinder2->position();
    Vector cylinder_top = puz_cylinder3->position();
    Vector ridd = riddleQ->position();

    //Set up the display to the user
    Vector newCoor = Vector(float((camerapos.x-box.x)), float((camerapos.y-box.y)), float((camerapos.z-box.z)));
    Vector newCoor2 = Vector(float((camerapos.x-cylinder_base.x)), float((camerapos.y-cylinder_base.y)), float((camerapos.z-cylinder_base.z)));
    Vector newCoor3 = Vector(float((camerapos.x-cylinder_middle.x)), float((camerapos.y-cylinder_middle.y)), float((camerapos.z-cylinder_middle.z)));
    Vector newCoor4 = Vector(float((camerapos.x-cylinder_top.x)), float((camerapos.y-cylinder_top.y)), float((camerapos.z-cylinder_top.z)));
    Vector newCoor5 = Vector((camerapos.x-ridd.x), (camerapos.y-ridd.y),(camerapos.z-ridd.z));

    //Leaves some space between the camera(user) and each object
    Vector heading = context->get(GF_CA_HEAD);
    Vector up = context->get(GF_CA_UP);
    Vector pos = context->get(GF_CA_POSN);
    Vector disp = 47 * heading; 
    Vector base = 40 * heading;
    Vector middle = 39 * heading;
    Vector top = 38 * heading;
    Matrix view;
    Vector lightPos;

    //Moves the lights from the mintor
    switch (section){
    case 0:		lightPos = Vector(puzzle1_light.x - box.x, puzzle1_light.y - box.y, puzzle1_light.z - box.z);		break;
    case 2:		lightPos = Vector(puzzle3_light.x - box.x, puzzle3_light.y - box.y, puzzle3_light.z - box.z);		break;
    }

    //Gameover
    //Displays the gameover screen to the user
    if (gameover){
        iTexture* temp = CreateTexture(L"gameover.bmp");
        rollRight->attach(temp);
        //Calculates the position/direction/up of the camera(user) and displays in front of it
        view = ::view(-pos, -heading, -up);
        view.m41 = camerapos.x+ disp.x;
        view.m42 = camerapos.y+ disp.y;
        view.m43 = camerapos.z+ disp.z;
        rollRight->setWorld(view);
        view.m41 = camerapos.x- base.x;
        view.m42 = camerapos.y- base.y;
        view.m43 = camerapos.z- base.z;
        puzzleLight4->setWorld(view);					
    }
    //Game Won
    //If the player completes all the puzzles then it will display the "THE END" screen followed by 
    //"SEVERAL DAYS LATER"
    else if (end){
        iTexture* temp;
        (!(after_end)) ? temp=CreateTexture(L"end.bmp") : temp=CreateTexture(L"days_later.bmp"); 
        rollRight->attach(temp);
        //Calculates the position/direction/up of the camera(user) and displays in front of it
        view = ::view(-pos, -heading, -up);
        view.m41 = camerapos.x+ disp.x;
        view.m42 = camerapos.y+ disp.y;
        view.m43 = camerapos.z+ disp.z;
        rollRight->setWorld(view);
        view.m41 = camerapos.x- base.x;
        view.m42 = camerapos.y- base.y;
        view.m43 = camerapos.z- base.z;
        puzzleLight4->setWorld(view);
    }
    //Puzzle/Riddle section
    else{
        if (section==0 || section==2){ //the puzzles
            //When the puzzle is not solved
            if (solved[section]==0){
                if (showpuzzle==0){
                    //Display the puzzle in front of the user
                    rollRight->translate((newCoor.x + disp.x), (newCoor.y), (newCoor.z+disp.z));
                    puz_cylinder1->translate((newCoor2.x + base.x), (newCoor2.y), (newCoor2.z+base.z));
                    puz_cylinder2->translate((newCoor3.x + middle.x), (newCoor3.y), (newCoor3.z+middle.z));
                    puz_cylinder3->translate((newCoor4.x + top.x), (newCoor4.y), (newCoor4.z+top.z));
                    showpuzzle=2;
                }
                else {
                    //Otherwise it puts the puzzle away
                    rollRight->translate(0,500,0);
                    puz_cylinder1->translate(0, 500, 0);
                    puz_cylinder2->translate(0,500, 0);
                    puz_cylinder3->translate(0, 500, 0);
                    showpuzzle=0;
                }
            }
            //When the puzzle is solved
            else {
                if (showpuzzle==0){
                    //Display the solved puzzle in front of the user
                    rollRight->translate((newCoor.x + disp.x), (newCoor.y), (newCoor.z+disp.z));
                    spinTop->translate((newCoor.x + disp.x), (newCoor.y), (newCoor.z+disp.z));
                    showpuzzle=2;
                }
                else {
                    //Otherwise it puts the puzzle away
                    rollRight->translate(0,500,0);
                    spinTop->translate(0, 500, 0);
                    showpuzzle=0;
                }
            }
        }
        //section1 - The Riddles
        else {
            //Shows the riddles
            iTexture* temp;
            if (!(answered[0])) { temp = CreateTexture(L"riddle1.bmp"); whichRiddle=1; }
            else if (!(answered[1])) { temp = CreateTexture(L"riddle2.bmp"); whichRiddle=2; }
            else if (!(answered[2])) { temp = CreateTexture(L"riddle3.bmp"); whichRiddle=3; }
            else if (answered[0] && answered[1] && answered[2]) { temp = CreateTexture(L"riddleu.bmp"); whichRiddle=4; solved[section]=1; }
            riddleQ->attach(temp);


            //Displays the riddle question/completion to the user
            if (showpuzzle==0){
                rollRight->translate((newCoor.x + disp.x), (newCoor.y), (newCoor.z+disp.z));
                riddleQ->translate((newCoor5.x + base.x), (newCoor5.y), (newCoor5.z+base.z));
                showpuzzle=2;
            }
            //Puts the riddle question aways
            else {
                rollRight->translate(0,500,0);
                riddleQ->translate(0,500,0);
                showpuzzle=0;
            }
        }

        //Lights section
        switch (section){
        case 0:
            //If the puzzle is in front of the user then reobtain the puzzle position and translates the lights so that
            //the user is able to read the puzzle
            if (showpuzzle==2){
                box = rollRight->position();
                lightPos = Vector(puzzle1_light.x - box.x, puzzle1_light.y - box.y, puzzle1_light.z - box.z);
                puzzleLight1->translate(-(lightPos.x + base.x), -lightPos.y, -(lightPos.z + base.z));
            }
            //Otherwise it would moved it back to where the monitor is located
            else       puzzleLight1->translate((lightPos.x + base.x), lightPos.y, (lightPos.z + base.z));
            break;
        case 1:
            //If the riddle is in front of the user then reobtain the puzzle position and translates the lights so that
            //the user is able to read the riddle
            if (showpuzzle==2){
                if (puzzle2_light.x != lastRiddle.x ){ //If the light position was not the same as before then it would move
                    lastRiddle = puzzle2_light;
                    lightPos=Vector(puzzle2_light.x - camerapos.x, puzzle2_light.y - camerapos.y, puzzle2_light.z - camerapos.z);
                    puzzleLight2->translate(-(lightPos.x + base.x), -lightPos.y, -(lightPos.z + base.z));
                }
            }
            //Moves the light position to where the monitor is located
            else  {
                lightPos=Vector(puzzle2_light.x - camerapos.x, puzzle2_light.y - camerapos.y, puzzle2_light.z - camerapos.z);
                puzzleLight2->translate((lightPos.x - base.x), lightPos.y, (lightPos.z - base.z));
            }

            break;
        case 2:
            //If the puzzle is in front of the user then reobtain the puzzle position and translates the lights so that
            //the user is able to read the puzzle
            if (showpuzzle==2){
                box = rollRight->position();
                lightPos = Vector(puzzle3_light.x - box.x, puzzle3_light.y - box.y, puzzle3_light.z - box.z);
                puzzleLight3->translate(-(lightPos.x + base.x), -lightPos.y, -(lightPos.z + base.z));
            }
            //Otherwise it would moved it back to where the monitor is located
            else    puzzleLight3->translate((lightPos.x + base.x), lightPos.y, (lightPos.z + base.z));
            break;
        }
    }//End of riddle section
} //End of setupPuzzle

// rotatePuzzle method
// rotates the selected puzzle section by 90 degrees clockwise
void Design::rotatePuzzle(int changeOne) {

    puz_cylinder1->rotatey(ToRadians(-120));
    puz_cylinder2->rotatey(ToRadians(-120));
    puz_cylinder3->rotatey(ToRadians(-120));           

    switch(changeOne){
    case 0:   puz_cylinder1->rotatez(ToRadians(90));   randompos[0]+=1;     break;
    case 1:   puz_cylinder2->rotatez(ToRadians(90));   randompos[1]+=1;     break;
    case 2:   puz_cylinder3->rotatez(ToRadians(90));   randompos[2]+=1;     break;           
    }

    puz_cylinder1->rotatey(ToRadians(120));
    puz_cylinder2->rotatey(ToRadians(120));
    puz_cylinder3->rotatey(ToRadians(120));

    for (int i=0;i<3;i++){
        if (randompos[i]>3)    randompos[i]=0;
    }
}


// update updates the position and orientation of each object for time "now"
// according to the keys pressed - the coordinate system for user actions is
// the left handed system with the x axis to the right, the y axis to the top 
// and the z axis into the screen
// Displays the game won/lost screen and manages the user input
//
void Design::update(int now) {

    int delta;
    static unsigned lastnow = 0;
    static unsigned elap =0;
    unsigned int rightnow = 1000;
    rightnow = now;
    delta  = now - lastUpdate;

    Vector camerapos = camera->position();

    ///////////////
    // Game Over //
    ///////////////
    if (!(gameover)){
        if (elapsed==0){
            elap=rightnow;
        }
        elapsed++;
        //If the player has taken more than 5 minutes to finish all the puzzles and
        //exited the level then player will die
        if ((rightnow - elap) >= 500000){
            context->set(LOCK_MOVE,0,true); //Locks the movements
            gameover=true;
            puzzleLight4->toggle(now);
            setupPuzzle(camerapos);
            game_over->toggle(now);
            //Turns off other sounds
            background->toggle(now);
            locall->toggle(now);
            localr->toggle(now);
            end=true;
        }

    }

    //Emergency light will turn off/on every 100 milliseconds
    if (rightnow - lastnow >= 100)     emergencyLight->toggle(now);

    //If player solved all the puzzles and reaches the exit door then it will display the "THE END" message followed by "SEVERAL DAYS LATER"
    if((camerapos.x >=399 && camerapos.x <= 478) && (camerapos.z <=-329 && camerapos.z >=-353) && (solved[0] ==1 && solved[1] ==1 && solved[2] == 1)){
        if(!(end)){
            end = true;
            context->set(LOCK_MOVE,0,true);
            locall->toggle(now);
            localr->toggle(now);
            setupPuzzle(camerapos);
            puzzleLight4->toggle(now);
            lastnow = now;
        }
        if(end && (!(after_end)) && ((rightnow - lastnow) >= AFTEREND)){
            after_end = true;
            setupPuzzle(camerapos);
            background->toggle(now);
            discrete->toggle(now);
            lastnow=now;
        }
        //Stops the audio track from looping
        if (after_end && (rightnow-lastnow)>22700 && !after){
            discrete->toggle(now);
            after=true;
        }
    }


    // lighting ------------------------------------------------------------------
    if (context->pressed(TOGGLE_FLASH))    topLight->toggle(now);
    // ---------------------------------------------------------------------------


    // Interaction ------------------------------------------------------------------
    // When the player presses R then it will display/hide the puzzle
    // ------------------------------------------------------------------------------
    if (context->pressed(DISPLAY_BOX)){
        if(rightnow - lastnow >= MIN_ROTATE){
            lastnow = rightnow;

            //Puzzle#1
            if((camerapos.x >= 137 && camerapos.x <= 175) && (camerapos.z >= 48 && camerapos.z <=75)){
                if(!end){
                    //Unlocks/Locks the movement
                    if(context->move_locked()) context->set(LOCK_MOVE,0,false);
                    else                       context->set(LOCK_MOVE,0,true);
                }
                //It player went to do another puzzle then reattach the texture
                if(baby == true){
                    iTexture* zombie_base = CreateTexture(L"zombie_1.bmp");
                    iTexture* zombie_middle = CreateTexture(L"zombie_2.bmp");
                    iTexture* zombie_top = CreateTexture(L"zombie_3.bmp");
                    iTexture* solution1 = CreateTexture(L"solved_1.bmp");
                    puz_cylinder1->attach(zombie_base);
                    puz_cylinder2->attach(zombie_middle);
                    puz_cylinder3->attach(zombie_top);
                    spinTop->attach(solution1);
                    baby = false;
                    pyramid = true;
                }
                section=0;
                //If this is the first time the player opens the puzzle then randomize the puzzle
                if (firstTime[section]){
                    int post =0;
                    int part =0;
                    for (int i=0;i<3;i++){
                        part=i;
                        post = (rand() % 4) + 1;
                        for (int z=0;z<post;z++){ rotatePuzzle(part); }
                    }
                    firstTime[section]=false;
                    showpuzzle=0;
                }
                //Sets up the puzzle
                setupPuzzle(camerapos);
            }// End of Puzzle#1

            //Puzzle#3
            else if((camerapos.x >= 1119 && camerapos.x <= 1176) && (camerapos.z >= 1854 && camerapos.z <=1877)){
                if(!end){
                    //Unlocks/Locks the movement
                    if(context->move_locked())  context->set(LOCK_MOVE,0,false);
                    else                        context->set(LOCK_MOVE,0,true);
                }
                //It player went to do another puzzle then reattach the texture
                if(pyramid == true){
                    iTexture* temp_base = CreateTexture(L"pyramid.bmp");
                    iTexture* temp_middle = CreateTexture(L"pyramid_2.bmp");
                    iTexture* temp_top = CreateTexture(L"pyramid_3.bmp");
                    iTexture* solution1 = CreateTexture(L"solution2.bmp");
                    puz_cylinder1->attach(temp_base);
                    puz_cylinder2->attach(temp_middle);
                    puz_cylinder3->attach(temp_top);
                    spinTop->attach(solution1);
                    baby = true;
                    pyramid = false;
                }
                section=2;
                //If this is the first time the player opens the puzzle then randomize the puzzle
                if (firstTime[section]){
                    int post =0;
                    int part =0;
                    for (int i=0;i<3;i++){
                        part=i;
                        post = (rand() % 4) + 1;
                        for (int z=0;z<post;z++) {  rotatePuzzle(part); }
                    }
                    firstTime[section]=false;
                    showpuzzle=0;
                }
                //Sets up the puzzle
                setupPuzzle(camerapos);

            }//End of Puzzle#3
            //Riddles
            else if((camerapos.x >= 158 && camerapos.x <= 229) && (camerapos.z >= 950 && camerapos.z <=1008)){
                if(!end){
                    //Unlocks/Locks the movement
                    if(context->move_locked())  context->set(LOCK_MOVE,0,false);
                    else                        context->set(LOCK_MOVE,0,true);
                }
                section=1;
                if (firstTime[section]) showpuzzle=0;
                setupPuzzle(camerapos);
                firstTime[section]=false;
            }
        }
    } //End of DISPLAY_BOX

    // Checks if the user is at the position and pressed the right answer key
    // Press 2 ------------------------------------------------------------------
    // --------------------------------------------------------------------------
    if (context->pressed(OPTION_TWO)){
        if((camerapos.x >= 158 && camerapos.x <= 229) && (camerapos.z >= 950 && camerapos.z <=1008) && showpuzzle==2 && whichRiddle==1){  
            answered[0]=true;    
            showpuzzle=0;
            setupPuzzle(camerapos); 
        }
    } //End of OPTION_TWO

    // Press 1 ------------------------------------------------------------------
    //---------------------------------------------------------------------------
    if (context->pressed(OPTION_ONE)){
        if((camerapos.x >= 158 && camerapos.x <= 229) && (camerapos.z >= 950 && camerapos.z <=1008) && showpuzzle==2 && whichRiddle==2){  
            answered[1]=true;    
            showpuzzle=0;
            setupPuzzle(camerapos); 
        }
    } //End of OPTION_ONE

    // Press 3 ------------------------------------------------------------------
    // --------------------------------------------------------------------------
    if (context->pressed(OPTION_THREE)){
        if((camerapos.x >= 158 && camerapos.x <= 229) && (camerapos.z >= 950 && camerapos.z <=1008) && showpuzzle==2 && whichRiddle==3){  
            answered[2]=true;    
            showpuzzle=0;
            setupPuzzle(camerapos); 
        }
    } //End of OPTION_THREE

    // Changes the puzzle object section for rotation
    // Press Y ------------------------------------------------------------------
    // --------------------------------------------------------------------------
    if (context->pressed(CHANGE_BOX)){
        if(rightnow - lastnow >= MIN_ROTATE){
            lastnow = rightnow;
            changeOne++;
            if (changeOne >= 3) changeOne=0;
        }
    } //End of CHANGE_BOX

    // Rotates the puzzle object section
    // Press T ------------------------------------------------------------------
    // --------------------------------------------------------------------------
    if (context->pressed(MOVE_BOX)){

        if(rightnow - lastnow >= MIN_ROTATE){
            lastnow = rightnow;

            if (solved[section]==0){
                //Rotates object
                rotatePuzzle(changeOne);
                //Checks if the puzzle is done
                if (randompos[0]==0 && randompos[1]==0 && randompos[2]==0){
                    //Displays the solved message and hides the cylinder objects
                    Vector box = spinTop->position();
                    Vector newCoor = Vector(float((camerapos.x-box.x)), float((camerapos.y-box.y)), float((camerapos.z-box.z)));
                    Vector heading = context->get(GF_CA_HEAD);
                    Vector disp = 40 * heading; 

                    puz_cylinder1->translate(0, 500, 0);
                    puz_cylinder2->translate(0, 500, 0);
                    puz_cylinder3->translate(0, 500, 0);
                    spinTop->translate((newCoor.x + disp.x), (newCoor.y), (newCoor.z+disp.z));
                    solved[section]=1;
                }
            }
        }
    } //End of MOVE_BOX

    // reference time
    lastUpdate = now;
}
