#ifndef _DESIGN_H_
#define _DESIGN_H_

/* Design Declaration - Model Branch
*
* Design.h
* fwk4gps version 1.0
* gam666/dps901
* August 30 2010
* copyright (c) 2010 Chris Szalwinski 
* distributed under TPL - see ../Licenses.txt
*/

#include "iDesign.h" // for the Design Interface

//-------------------------------- Design -------------------------------------
//
// The Design Component holds the game design proper
//
class iObject;
class iLight;
class iSound;
class iText;

class Design : public iDesign {

    bool pyramid;
    bool baby;
    iContext* context;         // points to the Context object
    iCamera* camera; 
    iSound*   background;      // points to the background sound
    iSound*   discrete;        // points to the discrete sound
    iSound*   locall;          // points to local sound on the left
    iSound*   localr;          // points to local sound on the right
	iSound*   game_over;

    iLight*   puzzleLight1;      // points to the point light
    iLight*   spotLight;       // points to the flashlight that is attached to the camera
    iLight*   puzzleLight2;    // points to the directional light
    iLight*   puzzleLight3;      // points to the cloned light
    iLight*   puzzleLight4;     // points to the light used for gameover and the end
    iLight*   topLight;        // points to the spot on the top box
    iLight*   distantLight;    // points to the global world light
    iLight* emergencyLight;    // points to the flickering red light on the exit sign

    iObject*  puz_cylinder1;   // points to base puzzle cylinder
    iObject*  puz_cylinder2;   // points to middle puzzle cylinder
    iObject*  puz_cylinder3;   // points to top puzzle cylinder
    iObject*  rollRight;       // points to the box used to display the puzzle desktop, the gameover screen and the end screen
    iObject*  spinTop;         // points to top spinner

    iObject*  riddleQ;         // points to the box used to display the riddles


    int changeOne;  //used to rotate the puzzle cylinders
    int showpuzzle; // used to determine if the puzzle should be showed or hidden
    int randompos[3]; //used to generate the degrees that the puzzle cylinders would be rotated
    int solved[3];    // used to keep track of which puzzles have been solved
    int section;      // used determine which puzzle you are attempting
    bool firstTime[3]; //used to keep track of the first time each puzzle is initiated
    bool answered[3];  //used to keep track of which riddle questions have been answered correctly
    bool end;          //keeps track of wether the game has ended or not
    bool after_end;    //keeps track of wether to play the after end sound effect and display the after end screen
    int whichRiddle;   //keeps track of which riddle question you are currently on
    int       lastUpdate;      // time that the model was last updated
    Vector puzzle1_light,puzzle2_light,puzzle3_light,puzzle4_light; //used to store the position of all of the puzzle lights
    Vector lastRiddle;
    bool after;        //used to stop the final sound effect so that it doesnt loop infinately
    int elapsed;       //used to keep track of the time that has elapsed since the game started so that it can be used for gameover
    bool gameover;     //keeps track of if you have triggered gameover or not
    Design(iContext* c);
    Design(const Design& s);            // prevents copying
    Design& operator=(const Design& s); // prevents assignment
    virtual ~Design() {}

public:
    friend iDesign* CreateDesign(iContext* c);
    // initialization functions
    void   initialize(int now);
    void   reset(int now);
    void   restore(int now);
    // execution function
    void   update(int now);
    void   setupPuzzle(Vector camerapos);
    void rotatePuzzle(int changeOne);
    // termination functions
    void   Delete() const { delete this; }
};

#endif
